function invasion_P(K, dx, iterations, maxT, n0, run_index)
% Calculates, and saves, the invasion probability based on a sequence of
% invasion attempts.
% This file was used to generate Fig. 2
% K: Carrying capacity of resident population
% dx: The shift in theta of the invading mutant
% Iterations: The number of invasion attempts
% maxT: maximal simulation time. If no fixation has occurred, the result is
% 'ambiguous'
% n0: Initial population size
% run_index: File id for output (allows parallell processing)

rand('state',fix(sum(1000*clock)+1000*run_index))
randn('state',fix(sum(2000*clock)+1000*run_index))

pars = getPars(K,maxT);
if K>=1000 || K==100
    file_name = sprintf('IS_K%d_n%d.mat', log10(K), n0);
else
    file_name = sprintf('IS_K%d_n%d.mat', K, n0);
end
% Load initial state:
load(file_name) % loads 'env' and 'startPop'
dt = 0.01;

% introduce mutant:
startPop.n = startPop.n-1; % remove a resident
startPop.n(2) = 1;
startPop.x(2) = startPop.x+dx;

invasions = 0;
ambiguous = 0;
tt = [];
for i=1:iterations
	[success,t] = sim_invasion_attempt(startPop, env, dt, pars, maxT);
	if isnan(success)
		ambiguous = ambiguous+1;
	elseif success
		invasions = invasions + 1;
		tt = [tt t]; % collect invasion times
	end
	drawnow
end
p = invasions / iterations;
n = startPop.ntot;
if K>=1000 || K==100
    file_name = sprintf('P_K%d_n%0.1f_dx%d_%d.mat',log10(K),n0/K*10,round(abs(dx)*1000),run_index);
else
    file_name = sprintf('P_K%d_n%0.1f_dx%d_%d.mat',K,n0/K*10,round(abs(dx)*1000),run_index);
end
%file_name = sprintf('invP_K%d_n0%d_%d.mat', K, n0, run_index);
save(file_name,'p','tt','ambiguous', 'iterations','maxT','pars','startPop','env','dt')



function [fixation, invasion_duration] = sim_invasion_attempt(pop, env, dt, pars, maxT)
epsilon = getEnvironmentAt(env,0);
t_now = 0;
invasion_duration = NaN;
n = pop.n;
x = pop.x;
b0 = pars.d0 + pars.r;
d0 = pars.d0;
r = pars.r;
K = pars.K; % this is a vector
envT1 = env.T1;
delta_epsilon = (env.v1-env.v0)*dt/env.dt;
epsilon = epsilon - delta_epsilon; % correct for first update below
while t_now < maxT
	% Update environment:
	if t_now>envT1
		[epsilon,env] = getEnvironmentAt(env,t_now);
		envT1 = env.T1;
		delta_epsilon = (env.v1-env.v0)*dt/env.dt;
	else
		epsilon = epsilon + delta_epsilon;%envv0 + (envv1-envv0)*(t_now-envT0)/envdt;
	end

	%[b,d] = birth_death(x,n,pars,eps);
	b = b0 + epsilon;
	if b<=0
		b=0;
		births = 0;
	else
		B = b.*n*dt;
		births = zeros(1,2);
		births(1) = poissrndApprSingle(B(1));
		births(2) = poissrndApprSingle(B(2));
	end

	d = d0 + r*(sum(n)/K).^x;
	D = d.*n*dt;
	deaths = zeros(1,2);
	deaths(1) = poissrndApprSingle(D(1));
	deaths(2) = poissrndApprSingle(D(2));

	n = n + births - deaths;
	if any(n<=0)
		fixation = n(1)<=0 && n(2)>0;
		if fixation
			invasion_duration = t_now;
			disp(['Fixation at t=' num2str(t_now)])
		end
		return
	end
	t_now = t_now + dt;
	%drawnow
end
fixation = NaN;%ambiguous
